package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.common.DictTypeTree
import com.gxzc.zen.api.sys.mapper.SysDictTypeMapper
import com.gxzc.zen.api.sys.model.SysDictType
import com.gxzc.zen.api.sys.service.ISysDictTypeService
import com.gxzc.zen.api.sys.service.ISysDictValueService
import com.gxzc.zen.common.contants.ZenConstants
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.TreeUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.springframework.beans.BeanUtils
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * <p>
 * 字典结构树 服务实现类
 * </p>
 *
 * @author NorthLan
 * @since 2018-04-28
 */
@Service
class SysDictTypeServiceImpl : ServiceImpl<SysDictTypeMapper, SysDictType>(), ISysDictTypeService {
    @Autowired
    private lateinit var dictValueService: ISysDictValueService

    private fun getTree(list: List<SysDictType>): MutableList<DictTypeTree> {
        val result = mutableListOf<DictTypeTree>()
        list.forEach {
            val node = DictTypeTree()
            BeanUtils.copyProperties(it, node)
            result.add(node)
        }
        return TreeUtil.build(result, ZenConstants.TREE_ROOT_PID)
    }

    override fun getDictTypeTree(): MutableList<DictTypeTree> {
        return getTree(baseMapper.selectList(null))
    }

    @ZenTransactional
    override fun createDictType(entity: SysDictType): SysDictType {
        // 先新增一条数据 再根据此数据更新对应的path
        if (baseMapper.insert(entity) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        if (baseMapper.updatePath(entity.parentId!!, entity.id!!) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        return entity
    }

    override fun updateDictType(entity: SysDictType): SysDictType {
        if (baseMapper.updateById(entity) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_UPDATE_ERROR)
        }
        return entity
    }

    @ZenTransactional
    override fun deleteDictType(id: Long?) {
        if (id == null) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        // 先查询现有的数据 (path)
        val stored = baseMapper.selectById(id) ?: throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        if (baseMapper.physicalDelete(EntityWrapper<SysDictType>().like("path", "${stored.path!!}%")) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        dictValueService.deleteByTypeId(id)
    }
}
